from fastapi import FastAPI, Response, Body, Depends
from fastapi.responses import ORJSONResponse
import json
from fastapi import HTTPException
from typing import Union, List
import requests
from sredis.sredis import * 
from sredis.models import * 
import logging
from core.models import User, DataCenter, Token
from uuid import uuid4
from core.auth import signJWT, hash_pass
from core.bearer import get_current_user

logging.config.fileConfig('logging.conf', disable_existing_loggers=False)

# get root logger
logger = logging.getLogger(__name__)
requests.packages.urllib3.disable_warnings() 

app = FastAPI()
add_datacenter("https://kappa1.fured.cloud.bme.hu")
add_datacenter("https://kappa2.fured.cloud.bme.hu")
add_datacenter("https://kappa3.fured.cloud.bme.hu")


@app.post("/user/signup", tags=["user"])
async def create_user(user: User = Body(...)):
    user = PUser(
        username=user.username,
        email=user.email,
        password=hash_pass(user.password)
    )
    user.save()
    return signJWT(user.username)

@app.post("/user/login", tags=["user"])
async def user_login(user: UserLoginSchema = Body(...)):
    if check_user(user):
        return signJWT(user.username)
    return {
        "error": "Wrong login details!"
    }

def _proxy_datacenters(serverpath: str, username, method="GET", balancer_fun = rr_get):
    server = balancer_fun()
    token = get_datacenter_token(username, server)
    url=f"{server}/{serverpath}"
    logger.debug("Req: " + url)
    t_resp = requests.request(
        method=method,
        url=url,
        allow_redirects=False, verify=False, 
        headers={
            'Authorization': token
        }
    )
    if t_resp.status_code / 100 != 2:
        raise HTTPException(status_code=t_resp.status_code, detail="Remote server error")
    response = ORJSONResponse(
        json.loads(t_resp.content), 
        status_code=t_resp.status_code
    )
    return response

@app.get("/lb/{server_path:path}")
def proxy(
        server_path: str = "/",
        username = Depends(get_current_user)
    ):
    return _proxy_datacenters(server_path, username)

@app.post("/lb/{server_path:path}")
def proxy(
        server_path: str = "/",
        username = Depends(get_current_user)
    ):
    return _proxy_datacenters(server_path, username, method="POST")

@app.post("/add_datacenter/")
def create_datacenter(
        dc: DataCenter = None,
        username = Depends(get_current_user)
    ):
    add_datacenter(dc.name)
    return Response(status_code=201)

@app.post("/set_tokens/")
def set_tokens(
        tokens: List[Token] = None,
        username = Depends(get_current_user)
    ):
    for token in tokens:
        set_token(username, str(token.datacenter), str(token.token))
    return tokens
